# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

import argparse
import getpass
import importlib.util
import os
import os.path
import pkg_resources
import pyclip
import time
import subprocess
import sys
import _version

def main():
    arguments = parse_arguments()

    config = Config()

    file = _latest("/home/gnarwhal/Pictures/Screenshots")
    print(f"=> Selected file '{file}'")

    id = time.time_ns()
    (_, extension) = os.path.splitext(file)
    target_file_name = f"{id}{extension}"
    target_destination = f"{config.ssh_user}@{config.ssh_target}:{config.ssh_dir}/{target_file_name}"
    print(f"=> Uploading to '{target_destination}'")
    process = subprocess.run([
        "scp",
        f"-P {config.ssh_port}",
        file,
        target_destination,
    ])
    if process.returncode != 0:
        print("=> Error: failed to upload file")
        sys.exit(1)

    target_url = f"https://{config.ssh_target}/sshare/{target_file_name}"
    pyclip.copy(target_url)
    print(f"=> File available at '{target_url}'")

    sys.exit(0)


def parse_arguments():
    parser = argparse.ArgumentParser(
        prog = "SSHare",
        description = "Upload files to a server via ssh",
        formatter_class = argparse.RawDescriptionHelpFormatter,
        epilog =
"""
Configuration Files:

  Configuration files will be looked for in
  '$XDG_CONFIG_DIR/sshare/'. If XDG_DATA_DIR is not set,
  SSHare will fallback to '$HOME/.config/sshare/'.

  If it exists, `default.py' will be considered as config[0].
  Otherwise, all other configuration files will be ordered
  alphabetically.
"""
    )

    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"%(prog)s version {_version.version}",
    )
    parser.add_argument(
        "-l",
        "--list",
        action="store_const",
        help="List available configuration files",
    )
    parser.add_argument(
        "-c",
        "--config",
        help="Use the config at location CONFIG",
    )
    parser.add_argument(
        "-n",
        "--number",
        type=int,
        default=0,
        help="Use config number n (Default: 0)",
    )
    parser.add_argument(
        "-i",
        "--interactive",
        action="store_const",
        help="Run interactively",
    )
    arguments = parser.parse_args()

    return arguments


def _latest(directory, key=os.path.getmtime):
    files         = map(lambda file: f"{directory}/{file}", os.listdir(directory))
    selection     = next(files)
    selection_key = key(selection)
    for file in files:
        new_key = key(file)
        if new_key > selection_key:
            selection     = file
            selection_key = key
    return selection
    

class Config:
    def __init__(self):
        self.ssh_port   = 2222
        self.ssh_user   = getpass.getuser()
        self.ssh_target = "monodon.me"
        self.ssh_dir    = "/srv/gnarwhal/sshare"
